''*******************************
''* PLAY.spin v1.2       6/2016 *
''* Author: Werner L. Schneider *
''*******************************
''
''
VAR

    long cog                     ' cog flag/id

    'DO NOT REARRANGE LONGS
    long ay1_regs                ' 7 contiguous longs
    long buffA_ptr
    long buffB_ptr
    long buffC_ptr
    long flagA
    long flagB
    long flagC


OBJ

    ay1 : "AYcog"


PUB start(right, left)
    stop

    ay1_Regs := ay1.start(right, left)    ' Start the emulated AY chip in one cog
 
    ay1.resetRegisters                    ' Reset all AY registers
    ay1.setAllTone   
 
    cog := cognew(@entry, @ay1_regs) + 1


PUB stop
    if cog
        cogstop(cog~ - 1)
    longfill(@ay1_regs, 0, 7)


PUB tone(offs)

    if flagA == 0
        buffA_ptr := offs
        flagA := 1
    elseif flagB == 0
        buffB_ptr := offs
        flagB := 1
    elseif flagC == 0
        buffC_ptr := offs
        flagC := 1


PUB death(vol, tonea, toneb)

    ay1.setVolume(0, vol)
    ay1.setVolume(1, vol)

    ay1.setFreq(0, tonea)
    ay1.setFreq(1, toneb)

    waitcnt((clkfreq / 1000) * 20 + cnt)


PUB over(s)

    if s == 1
        ay1.setVolume(0, 15)
        ay1.setVolume(1, 15)
        ay1.setVolume(2, 15)
        ay1.setFreq(0, $4FF)
        ay1.setFreq(1, $4F0)
        ay1.setFreq(2, $4E8)

    else
        ay1.setVolume(0, 0)
        ay1.setVolume(1, 0)
        ay1.setVolume(2, 0)


DAT

'***********************************
'* Assembly language "play" driver *
'***********************************

                        org     
'
' Entry
'

entry                   movd      :par, #_ay1ptr
                        mov       t1, par
                        mov       t2, #7
:par                    mov       0, t1
                        add       :par, dlsb
                        add       t1, #4
                        djnz      t2, #:par

                        mov       _pd, pdelay
 
'*********************************************************************************************************

                        rdlong    _ay, _ay1ptr
                        mov       _ayA, _ay               ' AY Reg 0 Fine Tune Ch A
                        add       _ay, #2  
                        mov       _ayB, _ay               ' AY Reg 2 Fine Tune Ch B
                        add       _ay, #2  
                        mov       _ayC, _ay               ' AY Reg 4 Fine Tune Ch C
                        add       _ay, #4  
                        mov       _ayVolA, _ay            ' AY Reg 8 Volume Ch A
                        add       _ay, #1  
                        mov       _ayVolB, _ay            ' AY Reg 9 Volume Ch B
                        add       _ay, #1  
                        mov       _ayVolC, _ay            ' AY Reg 10 Volume Ch C

'*********************************************************************************************************

bigloop                 mov       _delay, cnt          
                        add       _delay, _pd

'*********************************************************************************************************

entry1                  rdlong    t1, _flagptrA          ' Read Ch A Flag
                        cmp       t1, #0            wz
             if_z       jmp       #entry2     
                        cmp       t1, #1            wz 
             if_z       add       t1, #1
             if_z       wrlong    t1, _flagptrA
             if_z       rdlong    _buffA, _buffptrA
                        rdlong    _tone, _buffA
                        add       _buffA, #4
                        mov       _vol, _tone
                        shr       _vol, #12
                        cmp       _tone, endmask    wz
             if_z       jmp       #readyA
                        and       _tone, tonemask
                        wrword    _tone, _ayA             ' AY Reg 0/1 Ch A Fine Tune/Coarse Tune
                        wrbyte    _vol, _ayVolA           ' AY Reg 8 Volume Ch A

'*********************************************************************************************************

entry2                  rdlong    t2, _flagptrB          ' Read Ch B Flag
                        cmp       t2, #0            wz
             if_z       jmp       #entry3     
                        cmp       t2, #1            wz 
             if_z       add       t2, #1
             if_z       wrlong    t2, _flagptrB
             if_z       rdlong    _buffB, _buffptrB
                        rdlong    _tone, _buffB  
                        add       _buffB, #4
                        mov       _vol, _tone
                        shr       _vol, #12
                        cmp       _tone, endmask    wz
             if_z       jmp       #readyB
                        and       _tone, tonemask
                        wrword    _tone, _ayB             ' AY Reg 2/3 Ch B Fine Tune/Coarse Tune
                        wrbyte    _vol, _ayVolB           ' AY Reg 9 Volume Ch B

'*********************************************************************************************************

entry3                  rdlong    t3, _flagptrC           ' Read Ch C Flag
                        cmp       t3, #0            wz
             if_z       jmp       #entry4     
                        cmp       t3, #1            wz 
             if_z       add       t3, #1
             if_z       wrlong    t3, _flagptrC
             if_z       rdlong    _buffC, _buffptrC
                        rdlong    _tone, _buffC  
                        add       _buffC, #4
                        mov       _vol, _tone
                        shr       _vol, #12
                        cmp       _tone, endmask    wz
             if_z       jmp       #readyC
                        and       _tone, tonemask
                        wrword    _tone, _ayC             ' AY Reg 4/5 Ch C Fine Tune/Coarse Tune
                        wrbyte    _vol, _ayVolC           ' AY Reg 10 Volume Ch C

'*********************************************************************************************************

entry4                  waitcnt   _delay, _pd        
                        jmp       #bigloop

'*********************************************************************************************************

readyA                  mov       t1, #0
                        wrlong    t1, _flagptrA
                        wrbyte    t1, _ayVolA             ' AY #1 Reg 8 Volume Ch A
                        jmp       #entry2

'*********************************************************************************************************

readyB                  mov       t2, #0
                        wrlong    t2, _flagptrB
                        wrbyte    t2, _ayVolB             ' AY #1 Reg 9 Volume Ch B
                        jmp       #entry3

'*********************************************************************************************************

readyC                  mov       t3, #0
                        wrlong    t3, _flagptrC
                        wrbyte    t3, _ayVolC             ' AY-Reg 10 Volume Ch C
                        jmp       #entry4

'*********************************************************************************************************



                        ' Data

dlsb                    long      1 << 9

pdelay                  long      80_000_000 / 25

endmask                 long      $FFFF
tonemask                long      $0FFF

'
' Uninitialized data
'
t1                      res       1
t2                      res       1
t3                      res       1
t4                      res       1

_delay                  res       1
_pd                     res       1

_ay1ptr                 res       1               ' 7 params
_buffptrA               res       1
_buffptrB               res       1
_buffptrC               res       1
_flagptrA               res       1
_flagptrB               res       1
_flagptrC               res       1


_ay                     res       1
_ayA                    res       1
_ayB                    res       1
_ayC                    res       1
_ayVolA                 res       1
_ayVolB                 res       1
_ayVolC                 res       1

_buffA                  res       1
_buffB                  res       1
_buffC                  res       1

_tone                   res       1

_vol                    res       1


                        fit     $1f0

{{

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}