//***************************
//* PALETTE.C 1.1 ws 4/2017 *
//***************************

#include <stdio.h>
#include <string.h>

long filesize(char *fname);
void waitkey(void);

int main(int argc, char **argv)
{
	FILE *in;
	FILE *out;
	unsigned int x, y, z;
    unsigned long fs;
    char outfile[20];
    char *filename, *name, *ext;
    unsigned char bmphead[54];
    unsigned char bmppal[16][4];
    unsigned long colors[16];
    unsigned long newcol[16];
	
    filename = strupr(argv[1]);

  	printf("PALETTE 1.1 ws 4/2017\n\n");

	if(argc < 2) {
    	printf("No File specified !!!\n\n");
        waitkey();
        return -1;
    }

    if(!strstr(filename,".BMP")) {  
    	printf("File has no .BMP Extension !!!\n\n");
        waitkey();
        return -2;
    }

	in = fopen(filename, "rb");
	if(!in) {
    	printf("File not found !!!\n\n");
        waitkey();
        return -3;
    }

    fs = filesize(filename);
    if((fs != 1142) && (fs != 1270)) {
    	printf("ERROR! Filesize must be:\n\n 1.142 Bytes (8x8x256 BMP) or 1.270 Bytes (16x12x256 BMP).\n\n");
        waitkey();
        return -4;
    }      
    
    name = strtok(filename, ".");
    ext = strtok(NULL, ".");
    strcpy(outfile,name);
    strcat(outfile,".PAL");

	for(x = 0 ; x<54 ; x++) {
		bmphead[x] = fgetc(in);
    }

    if((int)bmphead[0] != 66  || (int)bmphead[1] != 77) {
    	printf("File has no BMP-Header !!!\n\n");
        waitkey();
        return -5;
    }

    if(((int)bmphead[18] != 8) && ((int)bmphead[18] != 16)) {
    	printf("BMP must have 8 or 16 Pixel Width !!!\n\n");
        waitkey();
        return -6;
    }

    if(((int)bmphead[22] != 8) && ((int)bmphead[22] != 12)) {
    	printf("BMP must have 8 or 12 Pixel Height !!!\n\n");
        waitkey();
        return -7;
    }

	for(x = 0 ; x<16 ; x++) {
        for(y = 0 ; y<4 ; y++) {
            bmppal[x][y] = fgetc(in);
        }
    }
    fclose(in);

	for(x = 0 ; x<16 ; x++) {
        colors[x] = (bmppal[x][0] / 85) * 4;
        colors[x] += ((bmppal[x][1] / 85) << 4);  
        colors[x] += (((bmppal[x][2] / 85) * 4) << 4);
    }


/* Old Method
	for(x = 0 ; x<16 ; x++) {
        colors[x] = bmppal[x][2] << 16;
        colors[x] += bmppal[x][1] << 8;  
        colors[x] += bmppal[x][0];
    }

    y = 0;
	for(x = 0 ; x<16 ; x++) {
        if(colors[x] == 0x000000) {
            newcol[x] = 0x00;
        }
        else if(colors[x] == 0x000055) {
            newcol[x] = 0x04;
        }
        else if(colors[x] == 0x0000AA) {
            newcol[x] = 0x08;
        }
        else if(colors[x] == 0x0000FF) {
            newcol[x] = 0x0C;
        }
        else if(colors[x] == 0x005500) {
            newcol[x] = 0x10;
        }
        else if(colors[x] == 0x005555) {
            newcol[x] = 0x14;
        }
        else if(colors[x] == 0x0055AA) {
            newcol[x] = 0x18;
        }
        else if(colors[x] == 0x0055FF) {
            newcol[x] = 0x1C;
        }
        else if(colors[x] == 0x00AA00) {
            newcol[x] = 0x20;
        }
        else if(colors[x] == 0x00AA55) {
            newcol[x] = 0x24;
        }
        else if(colors[x] == 0x00AAAA) {
            newcol[x] = 0x28;
        }
        else if(colors[x] == 0x00AAFF) {
            newcol[x] = 0x2C;
        }
        else if(colors[x] == 0x00FF00) {
            newcol[x] = 0x30;
        }
        else if(colors[x] == 0x00FF55) {
            newcol[x] = 0x34;
        }
        else if(colors[x] == 0x00FFAA) {
            newcol[x] = 0x38;
        }
        else if(colors[x] == 0x00FFFF) {
            newcol[x] = 0x3C;
        }
        else if(colors[x] == 0x550000) {
            newcol[x] = 0x40;
        }
        else if(colors[x] == 0x550055) {
            newcol[x] = 0x44;
        }
        else if(colors[x] == 0x5500AA) {
            newcol[x] = 0x48;
        }
        else if(colors[x] == 0x5500FF) {
            newcol[x] = 0x4C;
        }
        else if(colors[x] == 0x555500) {
            newcol[x] = 0x50;
        }
        else if(colors[x] == 0x555555) {
            newcol[x] = 0x54;
        }
        else if(colors[x] == 0x5555AA) {
            newcol[x] = 0x58;
        }
        else if(colors[x] == 0x5555FF) {
            newcol[x] = 0x5C;
        }
        else if(colors[x] == 0x55AA00) {
            newcol[x] = 0x60;
        }
        else if(colors[x] == 0x55AA55) {
            newcol[x] = 0x64;
        }
        else if(colors[x] == 0x55AAAA) {
            newcol[x] = 0x68;
        }
        else if(colors[x] == 0x55AAFF) {
            newcol[x] = 0x6C;
        }
        else if(colors[x] == 0x55FF00) {
            newcol[x] = 0x70;
        }
        else if(colors[x] == 0x55FF55) {
            newcol[x] = 0x74;
        }
        else if(colors[x] == 0x55FFAA) {
            newcol[x] = 0x78;
        }
        else if(colors[x] == 0x55FFFF) {
            newcol[x] = 0x7C;
        }
        else if(colors[x] == 0xAA0000) {
            newcol[x] = 0x80;
        }
        else if(colors[x] == 0xAA0055) {
            newcol[x] = 0x84;
        }
        else if(colors[x] == 0xAA00AA) {
            newcol[x] = 0x88;
        }
        else if(colors[x] == 0xAA00FF) {
            newcol[x] = 0x8C;
        }
        else if(colors[x] == 0xAA5500) {
            newcol[x] = 0x90;
        }
        else if(colors[x] == 0xAA5555) {
            newcol[x] = 0x94;
        }
        else if(colors[x] == 0xAA55AA) {
            newcol[x] = 0x98;
        }
        else if(colors[x] == 0xAA55FF) {
            newcol[x] = 0x9C;
        }
        else if(colors[x] == 0xAAAA00) {
            newcol[x] = 0xA0;
        }
        else if(colors[x] == 0xAAAA55) {
            newcol[x] = 0xA4;
        }
        else if(colors[x] == 0xAAAAAA) {
            newcol[x] = 0xA8;
        }
        else if(colors[x] == 0xAAAAFF) {
            newcol[x] = 0xAC;
        }
        else if(colors[x] == 0xAAFF00) {
            newcol[x] = 0xB0;
        }
        else if(colors[x] == 0xAAFF55) {
            newcol[x] = 0xB4;
        }
        else if(colors[x] == 0xAAFFAA) {
            newcol[x] = 0xB8;
        }
        else if(colors[x] == 0xAAFFFF) {
            newcol[x] = 0xBC;
        }
        else if(colors[x] == 0xFF0000) {
            newcol[x] = 0xC0;
        }
        else if(colors[x] == 0xFF0055) {
            newcol[x] = 0xC4;
        }
        else if(colors[x] == 0xFF00AA) {
            newcol[x] = 0xC8;
        }
        else if(colors[x] == 0xFF00FF) {
            newcol[x] = 0xCC;
        }
        else if(colors[x] == 0xFF5500) {
            newcol[x] = 0xD0;
        }
        else if(colors[x] == 0xFF5555) {
            newcol[x] = 0xD4;
        }
        else if(colors[x] == 0xFF55AA) {
            newcol[x] = 0xD8;
        }
        else if(colors[x] == 0xFF55FF) {
            newcol[x] = 0xDC;
        }
        else if(colors[x] == 0xFFAA00) {
            newcol[x] = 0xE0;
        }
        else if(colors[x] == 0xFFAA55) {
            newcol[x] = 0xE4;
        }
        else if(colors[x] == 0xFFAAAA) {
            newcol[x] = 0xE8;
        }
        else if(colors[x] == 0xFFAAFF) {
            newcol[x] = 0xEC;
        }
        else if(colors[x] == 0xFFFF00) {
            newcol[x] = 0xF0;
        }
        else if(colors[x] == 0xFFFF55) {
            newcol[x] = 0xF4;
        }
        else if(colors[x] == 0xFFFFAA) {
            newcol[x] = 0xF8;
        }
        else if(colors[x] == 0xFFFFFF) {
            newcol[x] = 0xFC;
        }
    }


	for(x = 0 ; x<16 ; x++) {
        printf("%02X\r\n", newcol[x]);  
    }
*/


	out = fopen(outfile, "wb");
    for(x = 0 ; x<4 ; x++) {
        y = ((x+1) * 4) - 1;
        fprintf(out, "%s", "        long    $");  
        fprintf(out, "%02X_", colors[y--]);  
        fprintf(out, "%02X_", colors[y--]);  
        fprintf(out, "%02X_", colors[y--]);  
        fprintf(out, "%02X\r\n", colors[y]);  
    }    
    fclose(out);
  	printf("File %s is created.\n\n", outfile);
    waitkey();
    return 0;


}



long filesize(char *fname)
{
    FILE *fp ;
    long size =0L;     

    fp = fopen(fname, "rb");
    fseek(fp, 0, SEEK_END); 
    size = ftell(fp);       
    fclose(fp); 
    return size;
}     




void waitkey()
{
	int k;
    printf("Press any Key to continue...");
	k = getch();
    printf("\n");
}


